@@ -2,25 +2,22 @@ use crate::cache::StoreCache;
 use crate::data_loader_wrapper::DataLoaderWrapper;
 use ckb_db::iter::{DBIter, Direction, IteratorMode};
 use ckb_db_schema::{
-    Col, COLUMN_BLOCK_BODY, COLUMN_BLOCK_EPOCH, COLUMN_BLOCK_EXT, COLUMN_BLOCK_HEADER,
-    COLUMN_BLOCK_PROPOSAL_IDS, COLUMN_BLOCK_UNCLE, COLUMN_CELL, COLUMN_CELL_DATA,
-    COLUMN_CELL_DATA_HASH, COLUMN_EPOCH, COLUMN_INDEX, COLUMN_META, COLUMN_TRANSACTION_INFO,
-    COLUMN_UNCLES, META_CURRENT_EPOCH_KEY, META_TIP_HEADER_KEY,
+    Col, COLUMN_BLOCK_BODY, COLUMN_BLOCK_EPOCH, COLUMN_BLOCK_EXT, COLUMN_BLOCK_EXTENSION,
+    COLUMN_BLOCK_HEADER, COLUMN_BLOCK_PROPOSAL_IDS, COLUMN_BLOCK_UNCLE, COLUMN_CELL,
+    COLUMN_CELL_DATA, COLUMN_CELL_DATA_HASH, COLUMN_EPOCH, COLUMN_INDEX, COLUMN_META,
+    COLUMN_TRANSACTION_INFO, COLUMN_UNCLES, META_CURRENT_EPOCH_KEY, META_TIP_HEADER_KEY,
 };
 use ckb_freezer::Freezer;
 use ckb_types::{
     bytes::Bytes,
     core::{
-        cell::{CellChecker, CellMeta, CellProvider, CellStatus},
-        BlockExt, BlockNumber, BlockView, EpochExt, EpochNumber, HeaderView, TransactionInfo,
-        TransactionView, UncleBlockVecView,
+        cell::CellMeta, BlockExt, BlockNumber, BlockView, EpochExt, EpochNumber, HeaderView,
+        TransactionInfo, TransactionView, UncleBlockVecView,
     },
     packed::{self, OutPoint},
     prelude::*,
 };
 
-pub struct CellProviderWrapper<'a, S>(&'a S);
-
 /// TODO(doc): @quake
 pub trait ChainStore<'a>: Send + Sync + Sized {
     /// TODO(doc): @quake
@@ -33,10 +30,6 @@ pub trait ChainStore<'a>: Send + Sync + Sized {
     fn get(&'a self, col: Col, key: &[u8]) -> Option<Self::Vector>;
     /// TODO(doc): @quake
     fn get_iter(&self, col: Col, mode: IteratorMode) -> DBIter;
-    /// TODO(doc): @quake
-    fn cell_provider(&self) -> CellProviderWrapper<Self> {
-        CellProviderWrapper(self)
-    }
     /// Return the provider trait default implementation
     fn as_data_provider(&'a self) -> DataLoaderWrapper<'a, Self> {
         DataLoaderWrapper::new(self)
@@ -48,8 +41,9 @@ pub trait ChainStore<'a>: Send + Sync + Sized {
         if let Some(freezer) = self.freezer() {
             if header.number() > 0 && header.number() < freezer.number() {
                 let raw_block = freezer.retrieve(header.number()).expect("block frozen")?;
-                let raw_block =
-                    packed::BlockReader::from_slice_should_be_ok(&raw_block).to_entity();
+                let raw_block = packed::BlockReader::from_compatible_slice(&raw_block)
+                    .expect("checked data")
+                    .to_entity();
                 return Some(raw_block.into_view());
             }
         }
@@ -60,7 +54,14 @@ pub trait ChainStore<'a>: Send + Sync + Sized {
         let proposals = self
             .get_block_proposal_txs_ids(h)
             .expect("block proposal_ids must be stored");
-        Some(BlockView::new_unchecked(header, uncles, body, proposals))
+        let extension_opt = self.get_block_extension(h);
+
+        let block = if let Some(extension) = extension_opt {
+            BlockView::new_unchecked_with_extension(header, uncles, body, proposals, extension)
+        } else {
+            BlockView::new_unchecked(header, uncles, body, proposals)
+        };
+        Some(block)
     }
 
     /// Get header by block header hash
@@ -127,7 +128,18 @@ pub trait ChainStore<'a>: Send + Sync + Sized {
                     .to_entity()
             })
             .expect("block proposal_ids must be stored");
-        Some(BlockView::new_unchecked(header, uncles, body, proposals))
+
+        let extension_opt = self
+            .get(COLUMN_BLOCK_EXTENSION, hash.as_slice())
+            .map(|slice| packed::BytesReader::from_slice_should_be_ok(&slice.as_ref()).to_entity());
+
+        let block = if let Some(extension) = extension_opt {
+            BlockView::new_unchecked_with_extension(header, uncles, body, proposals, extension)
+        } else {
+            BlockView::new_unchecked(header, uncles, body, proposals)
+        };
+
+        Some(block)
     }
 
     /// Get all transaction-hashes in block body by block header hash
@@ -210,6 +222,24 @@ pub trait ChainStore<'a>: Send + Sync + Sized {
         }
     }
 
+    /// Get block extension by block header hash
+    fn get_block_extension(&'a self, hash: &packed::Byte32) -> Option<packed::Bytes> {
+        if let Some(cache) = self.cache() {
+            if let Some(data) = cache.block_extensions.lock().get(hash) {
+                return data.clone();
+            }
+        };
+
+        let ret = self
+            .get(COLUMN_BLOCK_EXTENSION, hash.as_slice())
+            .map(|slice| packed::BytesReader::from_slice_should_be_ok(&slice.as_ref()).to_entity());
+
+        if let Some(cache) = self.cache() {
+            cache.block_extensions.lock().put(hash.clone(), ret.clone());
+        }
+        ret
+    }
+
     /// Get block ext by block header hash
     fn get_block_ext(&'a self, block_hash: &packed::Byte32) -> Option<BlockExt> {
         self.get(COLUMN_BLOCK_EXT, block_hash.as_slice())
@@ -283,7 +313,8 @@ pub trait ChainStore<'a>: Send + Sync + Sized {
                 let raw_block = freezer
                     .retrieve(tx_info.block_number)
                     .expect("block frozen")?;
-                let raw_block_reader = packed::BlockReader::from_slice_should_be_ok(&raw_block);
+                let raw_block_reader =
+                    packed::BlockReader::from_compatible_slice(&raw_block).expect("checked data");
                 let tx_reader = raw_block_reader.transactions().get(tx_info.index)?;
                 return Some((tx_reader.to_entity().into_view(), tx_info));
             }
@@ -442,26 +473,13 @@ pub trait ChainStore<'a>: Send + Sync + Sized {
 
     /// Gets cellbase by block hash
     fn get_cellbase(&'a self, hash: &packed::Byte32) -> Option<TransactionView> {
-        if let Some(cache) = self.cache() {
-            if let Some(data) = cache.cellbase.lock().get(hash) {
-                return Some(data.clone());
-            }
-        };
         let key = packed::TransactionKey::new_builder()
             .block_hash(hash.to_owned())
             .build();
-        let ret = self.get(COLUMN_BLOCK_BODY, key.as_slice()).map(|slice| {
+        self.get(COLUMN_BLOCK_BODY, key.as_slice()).map(|slice| {
             let reader = packed::TransactionViewReader::from_slice_should_be_ok(&slice.as_ref());
             Unpack::<TransactionView>::unpack(&reader)
-        });
-        if let Some(cache) = self.cache() {
-            ret.map(|data| {
-                cache.cellbase.lock().put(hash.clone(), data.clone());
-                data
-            })
-        } else {
-            ret
-        }
+        })
     }
 
     /// TODO(doc): @quake
@@ -489,14 +507,27 @@ pub trait ChainStore<'a>: Send + Sync + Sized {
 
         let uncles = self.get_block_uncles(hash)?;
         let proposals = self.get_block_proposal_txs_ids(hash)?;
-        Some(
+        let extension_opt = self.get_block_extension(hash);
+
+        let block = if let Some(extension) = extension_opt {
+            packed::BlockV1::new_builder()
+                .header(header)
+                .uncles(uncles.data())
+                .transactions(transactions)
+                .proposals(proposals)
+                .extension(extension)
+                .build()
+                .as_v0()
+        } else {
             packed::Block::new_builder()
                 .header(header)
                 .uncles(uncles.data())
                 .transactions(transactions)
                 .proposals(proposals)
-                .build(),
-        )
+                .build()
+        };
+
+        Some(block)
     }
 
     /// TODO(doc): @quake
@@ -523,36 +554,3 @@ fn build_cell_meta_from_reader(out_point: OutPoint, reader: packed::CellEntryRea
         mem_cell_data_hash: None,
     }
 }
-
-impl<'a, S> CellProvider for CellProviderWrapper<'a, S>
-where
-    S: ChainStore<'a>,
-{
-    fn cell(&self, out_point: &OutPoint, eager_load: bool) -> CellStatus {
-        match self.0.get_cell(out_point) {
-            Some(mut cell_meta) => {
-                if eager_load {
-                    if let Some((data, data_hash)) = self.0.get_cell_data(out_point) {
-                        cell_meta.mem_cell_data = Some(data);
-                        cell_meta.mem_cell_data_hash = Some(data_hash);
-                    }
-                }
-                CellStatus::live_cell(cell_meta)
-            }
-            None => CellStatus::Unknown,
-        }
-    }
-}
-
-impl<'a, S> CellChecker for CellProviderWrapper<'a, S>
-where
-    S: ChainStore<'a>,
-{
-    fn is_live(&self, out_point: &OutPoint) -> Option<bool> {
-        if self.0.have_cell(out_point) {
-            Some(true)
-        } else {
-            None
-        }
-    }
-}
